@@ -1,12 +1,10 @@
-from __future__ import print_function
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
+# coding=utf-8
 from ctypes import *
 from gi.repository import GObject
+from gi.repository import GLib
+from gi.types import GObjectMeta
 import weakref
-from blueman.Functions import YELLOW, dprint
+import logging
 
 try:
     libpulse = CDLL("libpulse.so.0")
@@ -56,58 +54,58 @@ class PANotConnected(Exception):
     pass
 
 
-class pa_module_info(Structure):
+class PaModuleInfo(Structure):
     _fields_ = [("index", c_uint),
                 ("name", c_char_p),
                 ("argument", c_char_p),
                 ("n_used", c_int),
                 ("proplist", c_void_p),
-    ]
+                ]
 
 
-class pa_sample_spec(Structure):
+class PaSampleSpec(Structure):
     pass
 
 
-pa_sample_spec._fields_ = [
+PaSampleSpec._fields_ = [
     ('format', c_int),
     ('rate', c_uint32),
     ('channels', c_uint8),
 ]
 
 
-class pa_channel_map(Structure):
+class PaChannelMap(Structure):
     pass
 
 
-pa_channel_map._fields_ = [
+PaChannelMap._fields_ = [
     ('channels', c_uint8),
     ('map', c_int * 32),
 ]
 
 
-class pa_cvolume(Structure):
+class PaCvolume(Structure):
     pass
 
 
-pa_cvolume._fields_ = [
+PaCvolume._fields_ = [
     ('channels', c_uint8),
     ('values', c_uint32 * 32),
 ]
 
 
-class pa_source_info(Structure):
+class PaSourceInfo(Structure):
     pass
 
 
-pa_source_info._fields_ = [
+PaSourceInfo._fields_ = [
     ('name', c_char_p),
     ('index', c_uint32),
     ('description', c_char_p),
-    ('sample_spec', pa_sample_spec),
-    ('channel_map', pa_channel_map),
+    ('sample_spec', PaSampleSpec),
+    ('channel_map', PaChannelMap),
     ('owner_module', c_uint32),
-    ('volume', pa_cvolume),
+    ('volume', PaCvolume),
     ('mute', c_int),
     ('monitor_of_sink', c_uint32),
     ('monitor_of_sink_name', c_char_p),
@@ -126,18 +124,18 @@ pa_source_info._fields_ = [
 ]
 
 
-class pa_sink_info(Structure):
+class PaSinkInfo(Structure):
     pass
 
 
-pa_sink_info._fields_ = [
+PaSinkInfo._fields_ = [
     ('name', c_char_p),
     ('index', c_uint32),
     ('description', c_char_p),
-    ('sample_spec', pa_sample_spec),
-    ('channel_map', pa_channel_map),
+    ('sample_spec', PaSampleSpec),
+    ('channel_map', PaChannelMap),
     ('owner_module', c_uint32),
-    ('volume', pa_cvolume),
+    ('volume', PaCvolume),
     ('mute', c_int),
     ('monitor_of_sink', c_uint32),
     ('monitor_of_sink_name', c_char_p),
@@ -156,19 +154,19 @@ pa_sink_info._fields_ = [
 ]
 
 
-class pa_sink_input_info(Structure):
+class PaSinkInputInfo(Structure):
     pass
 
 
-pa_sink_input_info._fields_ = [
+PaSinkInputInfo._fields_ = [
     ('index', c_uint32),
     ('name', c_char_p),
     ('owner_module', c_uint32),
     ('client', c_uint32),
     ('sink', c_uint32),
-    ('sample_spec', pa_sample_spec),
-    ('channel_map', pa_channel_map),
-    ('volume', pa_cvolume),
+    ('sample_spec', PaSampleSpec),
+    ('channel_map', PaChannelMap),
+    ('volume', PaCvolume),
     ('buffer_usec', c_uint64),
     ('sink_usec', c_uint64),
     ('resample_method', c_char_p),
@@ -178,11 +176,11 @@ pa_sink_input_info._fields_ = [
 ]
 
 
-class pa_card_profile_info(Structure):
+class PaCardProfileInfo(Structure):
     pass
 
 
-pa_card_profile_info._fields_ = [
+PaCardProfileInfo._fields_ = [
     ('name', c_char_p),
     ('description', c_char_p),
     ('n_sinks', c_uint32),
@@ -191,18 +189,18 @@ pa_card_profile_info._fields_ = [
 ]
 
 
-class pa_card_info(Structure):
+class PaCardInfo(Structure):
     pass
 
 
-pa_card_info._fields_ = [
+PaCardInfo._fields_ = [
     ('index', c_uint32),
     ('name', c_char_p),
     ('owner_module', c_uint32),
     ('driver', c_char_p),
     ('n_profiles', c_uint32),
-    ('profiles', POINTER(pa_card_profile_info)),
-    ('active_profile', POINTER(pa_card_profile_info)),
+    ('profiles', POINTER(PaCardProfileInfo)),
+    ('active_profile', POINTER(PaCardProfileInfo)),
     ('proplist', c_void_p),
 ]
 pa_context_notify_cb_t = CFUNCTYPE(None, c_void_p, py_object)
@@ -210,13 +208,13 @@ pa_context_notify_cb_t = CFUNCTYPE(None, c_void_p, py_object)
 pa_context_index_cb_t = CFUNCTYPE(None, c_void_p, c_int, py_object)
 pa_context_success_cb_t = pa_context_index_cb_t
 
-pa_module_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(pa_module_info), c_int, py_object)
-pa_source_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(pa_source_info), c_int, py_object)
-pa_sink_input_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(pa_sink_input_info), c_int, py_object)
-pa_card_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(pa_card_info), c_int, py_object)
+pa_module_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(PaModuleInfo), c_int, py_object)
+pa_source_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(PaSourceInfo), c_int, py_object)
+pa_sink_input_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(PaSinkInputInfo), c_int, py_object)
+pa_card_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(PaCardInfo), c_int, py_object)
 pa_context_subscribe_cb_t = CFUNCTYPE(None, c_void_p, c_uint32, c_uint32, c_void_p)
 
-pa_sink_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(pa_sink_info), c_int, py_object)
+pa_sink_info_cb_t = CFUNCTYPE(None, c_void_p, POINTER(PaSinkInfo), c_int, py_object)
 
 pa_context_get_module_info_list = libpulse.pa_context_get_module_info_list
 pa_context_get_module_info_list.restype = c_void_p
@@ -327,11 +325,21 @@ pa_context_errno.restype = c_int
 pa_context_errno.argtypes = [c_void_p]
 
 
-class PulseAudioUtils(GObject.GObject):
+class PulseAudioUtilsMeta(GObjectMeta):
+    _instance = None
+
+    def __call__(cls, *args, **kwargs):
+        if not cls._instance:
+            cls._instance = super().__call__(*args, **kwargs)
+
+        return cls._instance
+
+
+class PulseAudioUtils(GObject.GObject, metaclass=PulseAudioUtilsMeta):
     __gsignals__ = {
-    str('connected'): (GObject.SignalFlags.NO_HOOKS, None, ()),
-    str('disconnected'): (GObject.SignalFlags.NO_HOOKS, None, ()),
-    str('event'): (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT)),
+        'connected': (GObject.SignalFlags.NO_HOOKS, None, ()),
+        'disconnected': (GObject.SignalFlags.NO_HOOKS, None, ()),
+        'event': (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT)),
     }
 
     def check_connected(self):
@@ -344,23 +352,23 @@ class PulseAudioUtils(GObject.GObject):
             return
 
         state = pa_context_get_state(pa_context)
-        dprint(state)
+        logging.info(state)
         if state == PA_CONTEXT_READY:
             self.connected = True
             self.emit("connected")
-            MASK = 0x0200 | 0x0010  # from enum pa_subscription_mask
+            mask = 0x0200 | 0x0010  # from enum pa_subscription_mask
 
-            self.simple_callback(lambda x: dprint(x),
+            self.simple_callback(lambda x: logging.info(x),
                                  pa_context_subscribe,
-                                 MASK)
+                                 mask)
         else:
             if self.connected:
                 self.emit("disconnected")
                 self.connected = False
 
         if self.prev_state == PA_CONTEXT_READY and state == PA_CONTEXT_FAILED:
-            dprint("Pulseaudio probably crashed, restarting in 5s")
-            GObject.timeout_add(5000, self.Connect)
+            logging.info("Pulseaudio probably crashed, restarting in 5s")
+            GLib.timeout_add(5000, self.connect_pulseaudio)
 
         self.prev_state = state
 
@@ -391,15 +399,15 @@ class PulseAudioUtils(GObject.GObject):
             info["handler"](None, True)
             pythonapi.Py_DecRef(py_object(info))
 
-    def __init_list_callback(self, function, cb_type, handler, *args):
+    def __init_list_callback(self, func, cb_type, handler, *args):
         info = {"cb_info": cb_type(self.__list_callback), "handler": handler}
         pythonapi.Py_IncRef(py_object(info))
 
         args += (info["cb_info"], py_object(info))
-        op = function(self.pa_context, *args)
+        op = func(self.pa_context, *args)
         pa_operation_unref(op)
 
-    def simple_callback(self, handler, function, *args):
+    def simple_callback(self, handler, func, *args):
 
         def wrapper(context, res, data):
             if handler:
@@ -410,14 +418,13 @@ class PulseAudioUtils(GObject.GObject):
         pythonapi.Py_IncRef(py_object(cb))
 
         args += (cb, py_object(cb))
-        op = function(self.pa_context, *args)
+        op = func(self.pa_context, *args)
         if not op:
-            dprint(YELLOW("Operation failed"))
-            print(function.__name__)
+            logging.info("Operation failed")
+            logging.error(func.__name__)
         pa_operation_unref(op)
 
-
-    def ListSources(self, callback):
+    def list_sources(self, callback):
         self.check_connected()
 
         data = {}
@@ -430,21 +437,19 @@ class PulseAudioUtils(GObject.GObject):
             props = self.__get_proplist(entry_info[0].proplist)
 
             data[entry_info[0].index] = {
-            "name": entry_info[0].name.decode("UTF-8"),
-            "proplist": props,
-            "description": entry_info[0].description.decode("UTF-8"),
-            "owner_module": entry_info[0].owner_module,
-            "driver": entry_info[0].driver.decode("UTF-8")
+                "name": entry_info[0].name.decode("UTF-8"),
+                "proplist": props,
+                "description": entry_info[0].description.decode("UTF-8"),
+                "owner_module": entry_info[0].owner_module,
+                "driver": entry_info[0].driver.decode("UTF-8")
             }
             if end:
                 callback(data)
 
-
         self.__init_list_callback(pa_context_get_source_info_list,
                                   pa_source_info_cb_t, handler)
 
-
-    def ListSinks(self, callback, id=None):
+    def list_sinks(self, callback, card_id=None):
         self.check_connected()
 
         data = {}
@@ -456,24 +461,24 @@ class PulseAudioUtils(GObject.GObject):
             props = self.__get_proplist(entry_info[0].proplist)
 
             data[entry_info[0].index] = {
-            "name": entry_info[0].name.decode("UTF-8"),
-            "proplist": props,
-            "description": entry_info[0].description.decode("UTF-8"),
-            "owner_module": entry_info[0].owner_module,
-            "driver": entry_info[0].driver.decode("UTF-8")
+                "name": entry_info[0].name.decode("UTF-8"),
+                "proplist": props,
+                "description": entry_info[0].description.decode("UTF-8"),
+                "owner_module": entry_info[0].owner_module,
+                "driver": entry_info[0].driver.decode("UTF-8")
             }
 
             if end:
                 callback(data)
 
-        if id != None:
+        if card_id is not None:
             self.__init_list_callback(pa_context_get_sink_info_list,
-                                      pa_sink_info_cb_t, handler, id)
+                                      pa_sink_info_cb_t, handler, card_id)
         else:
             self.__init_list_callback(pa_context_get_sink_info_list,
                                       pa_sink_info_cb_t, handler)
 
-    def ListSinkInputs(self, callback):
+    def list_sink_inputs(self, callback):
         self.check_connected()
 
         data = {}
@@ -486,24 +491,24 @@ class PulseAudioUtils(GObject.GObject):
             props = self.__get_proplist(entry_info[0].proplist)
 
             data[entry_info[0].index] = {
-            "name": entry_info[0].name.decode("UTF-8"),
-            "proplist": props,
-            "owner_module": entry_info[0].owner_module,
-            "sink": entry_info[0].sink,
-            "driver": entry_info[0].driver.decode("UTF-8")
+                "name": entry_info[0].name.decode("UTF-8"),
+                "proplist": props,
+                "owner_module": entry_info[0].owner_module,
+                "sink": entry_info[0].sink,
+                "driver": entry_info[0].driver.decode("UTF-8")
             }
 
         self.__init_list_callback(pa_context_get_sink_input_info_list,
                                   pa_sink_input_info_cb_t, handler)
 
-    def MoveSinkInput(self, input_id, sink_id, callback):
+    def move_sink_input(self, input_id, sink_id, callback):
         self.check_connected()
 
         self.simple_callback(callback,
                              pa_context_move_sink_input_by_index,
                              int(input_id), int(sink_id))
 
-    def SetDefaultSink(self, name, callback):
+    def set_default_sink(self, name, callback):
         self.check_connected()
 
         self.simple_callback(callback, pa_context_set_default_sink, name)
@@ -512,29 +517,29 @@ class PulseAudioUtils(GObject.GObject):
     def __card_info(self, card_info):
         props = self.__get_proplist(card_info[0].proplist)
         stuff = {
-        "name": card_info[0].name.decode("UTF-8"),
-        "proplist": props,
-        "owner_module": card_info[0].owner_module,
-        "driver": card_info[0].driver.decode("UTF-8"),
-        "index": card_info[0].index,
+            "name": card_info[0].name.decode("UTF-8"),
+            "proplist": props,
+            "owner_module": card_info[0].owner_module,
+            "driver": card_info[0].driver.decode("UTF-8"),
+            "index": card_info[0].index,
         }
-        l = []
+        profiles = []
         for i in range(0, card_info[0].n_profiles):
             x = {
-            "name": card_info[0].profiles[i].name.decode("UTF-8"),
-            "description": card_info[0].profiles[i].description.decode("UTF-8"),
-            "n_sinks": card_info[0].profiles[i].n_sinks,
-            "n_sources": card_info[0].profiles[i].n_sources,
-            "priority": card_info[0].profiles[i].priority,
+                "name": card_info[0].profiles[i].name.decode("UTF-8"),
+                "description": card_info[0].profiles[i].description.decode("UTF-8"),
+                "n_sinks": card_info[0].profiles[i].n_sinks,
+                "n_sources": card_info[0].profiles[i].n_sources,
+                "priority": card_info[0].profiles[i].priority,
             }
-            l.append(x)
+            profiles.append(x)
 
-        stuff["profiles"] = l
+        stuff["profiles"] = profiles
         stuff["active_profile"] = card_info[0].active_profile[0].name.decode("UTF-8")
 
         return stuff
 
-    def ListCards(self, callback):
+    def list_cards(self, callback):
         self.check_connected()
 
         data = {}
@@ -551,7 +556,7 @@ class PulseAudioUtils(GObject.GObject):
         self.__init_list_callback(pa_context_get_card_info_list,
                                   pa_card_info_cb_t, handler)
 
-    def GetCard(self, card, callback):
+    def get_card(self, card, callback):
         self.check_connected()
 
         def handler(entry_info, end):
@@ -560,7 +565,7 @@ class PulseAudioUtils(GObject.GObject):
 
             callback(self.__card_info(entry_info))
 
-        if hasattr(card, "format") and hasattr(card, "upper"):
+        if type(card) is str:
             fn = pa_context_get_card_info_by_name
         else:
             fn = pa_context_get_card_info_by_index
@@ -568,9 +573,9 @@ class PulseAudioUtils(GObject.GObject):
         self.__init_list_callback(fn,
                                   pa_card_info_cb_t, handler, card)
 
-    def SetCardProfile(self, card, profile, callback):
+    def set_card_profile(self, card, profile, callback):
         profile = profile.encode("UTF-8")
-        if hasattr(card, "format") and hasattr(card, "upper"):
+        if type(card) is str:
             card = card.encode("UTF-8")
             fn = pa_context_set_card_profile_by_name
         else:
@@ -578,9 +583,8 @@ class PulseAudioUtils(GObject.GObject):
 
         self.simple_callback(callback, fn, card, profile)
 
-    #### Module API #######
-    #from gi.repository import Gtk
-    def ListModules(self, callback):
+    # #### Module API ####
+    def list_module(self, callback):
 
         self.check_connected()
         data = {}
@@ -592,21 +596,21 @@ class PulseAudioUtils(GObject.GObject):
 
             props = self.__get_proplist(entry_info[0].proplist)
             data[entry_info[0].index] = {
-            "name": entry_info[0].name.decode("UTF-8"),
-            "argument": entry_info[0].argument.decode("UTF-8"),
-            "n_used": entry_info[0].n_used,
-            "proplist": props
+                "name": entry_info[0].name.decode("UTF-8"),
+                "argument": entry_info[0].argument.decode("UTF-8"),
+                "n_used": entry_info[0].n_used,
+                "proplist": props
             }
 
         self.__init_list_callback(pa_context_get_module_info_list,
                                   pa_module_info_cb_t, handler)
 
-    def UnloadModule(self, index, callback):
+    def unload_module(self, index, callback):
         self.check_connected()
 
         self.simple_callback(callback, pa_context_unload_module, index)
 
-    def LoadModule(self, name, args, callback):
+    def load_module(self, name, args, callback):
         self.check_connected()
 
         def handler(res):
@@ -615,39 +619,16 @@ class PulseAudioUtils(GObject.GObject):
             else:
                 callback(res)
 
-
         self.simple_callback(handler, pa_context_load_module, name, args)
 
     #####################
 
-    def GetVersion(self):
-        v = pa_get_library_version().decode("UTF-8")
-        try:
-            a = v.split("-")[0].split(".")
-            a = map(lambda x: int(x), a)
-        except:
-            a = (0, 0, 0)
-        return a
-
-    inst = None
-
-    def __new__(cls):
-        if cls.inst:
-            return PulseAudioUtils.inst
-        else:
-            return super(PulseAudioUtils, cls).__new__(cls)
-
-
     def __event_callback(self, context, event_type, idx, userdata):
-        dprint(event_type, idx)
+        logging.info("%s %s" % (event_type, idx))
         self.emit("event", event_type, idx)
 
     def __init__(self):
-        if PulseAudioUtils.inst != None:
-            return
-
-        PulseAudioUtils.inst = self
-        GObject.GObject.__init__(self)
+        super().__init__()
 
         self.event_cb = pa_context_subscribe_cb_t(self.__event_callback)
 
@@ -662,10 +643,11 @@ class PulseAudioUtils(GObject.GObject):
 
         self.prev_state = 0
 
-        self.Connect()
+        self.connect_pulseaudio()
 
+        weakref.finalize(self, self._on_delete)
 
-    def Connect(self):
+    def connect_pulseaudio(self):
         if not self.connected:
             if self.pa_context:
                 pa_context_unref(self.pa_context)
@@ -683,9 +665,8 @@ class PulseAudioUtils(GObject.GObject):
                                               self.event_cb,
                                               None)
 
-
-    def __del__(self):
-        dprint("Destroying PulseAudioUtils instance")
+    def _on_delete(self):
+        logging.info("Destroying PulseAudioUtils instance")
 
         pa_context_disconnect(self.pa_context)
         pa_context_unref(self.pa_context)
